/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.task.model;

import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.Point;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;
import java.util.Objects;
import java.util.logging.Logger;
import nl.cloudfarming.client.geoviewer.Geometrical;

/**
 * A treatment zone is a geographical area which is treated in a particular way.
 *
 * @author Merijn Zengers
 */
public class TreatmentZone implements Geometrical<Geometry>, Serializable {

    private Geometry geometry;
    public static final String PROP_GEOMETRY = "geometry";
    private transient PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);
    private static final Logger LOG = Logger.getLogger("nl.cloudfarming.client.task.model");
    private String name;
    private Double value;

    /**
     * Get the value of geometry
     *
     * @return the value of geometry
     */
    @Override
    public Geometry getGeometry() {
        return geometry;
    }
    
    @Override
    public Geometry getBoundingBox() {
        return (this.geometry!=null)?geometry.getBoundary():null;
    }    

    @Override
    public Point getCentroid() {
        return geometry.getCentroid();
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public Double getValue() {
        return value;
    }

    public void setValue(Double value) {
        this.value = value;
    }

    /**
     * Set the value of geometry
     *
     * @param geometry new value of geometry
     */
    public void setGeometry(Geometry geometry) {
        Geometry oldGeometry = this.geometry;
        this.geometry = geometry;
        getPropertyChangeSupport().firePropertyChange(PROP_GEOMETRY, oldGeometry, geometry);
    }

    PropertyChangeSupport getPropertyChangeSupport() {
        if (propertyChangeSupport == null) {
            propertyChangeSupport = new PropertyChangeSupport(this);
        }
        return propertyChangeSupport;
    }

    /**
     * Add PropertyChangeListener.
     *
     * @param listener
     */
    public void addPropertyChangeListener(PropertyChangeListener listener) {
        assert listener != null;
        getPropertyChangeSupport().addPropertyChangeListener(listener);
    }

    /**
     * Remove PropertyChangeListener.
     *
     * @param listener
     */
    public void removePropertyChangeListener(PropertyChangeListener listener) {
        getPropertyChangeSupport().removePropertyChangeListener(listener);
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final TreatmentZone other = (TreatmentZone) obj;
        if (!Objects.equals(this.geometry, other.geometry)) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 53 * hash + Objects.hashCode(this.geometry);
        return hash;
    }

    @Override
    public Geometry getRenderObject(Envelope envelope) {
        return geometry;
    }

    @Override
    public String getTooltipText() {
        return this.getValue() != null ? this.getValue().toString() : "Unknown value";
    }

    @Override
    public long getId() {
        return 0;
    }

    @Override
    public String getIconPath() {
        return null;
    }

    @Override
    public String getIconLabel() {
        return null;
    }
}
